local guildster, gTable = ...
local version = "0.0.2"

-- Saved Variables
if not GuildsterDB then
    GuildsterDB = {}
end
GDB = {
    sortOption = "name",
    initializedDate = nil
}

-- ScrollFrame Variables
gTable.numButtons = 3
gTable.buttonHeight = 0

-- Filter Variables
gTable.memberFilterLow = 0
gTable.memberFilterHigh = 999
gTable.updatedFilter = false
gTable.searchValue = nil
gTable.resultsFound = nil
gTable.selection = nil
gTable.filterTypes = {
    ["Questing"] = true,
    ["Dungeons"] = true,
    ["Raiding"] = true,
    ["PvP"] = true,
    ["RP"] = true,
    ["Weekends"] = true,
    ["Weekdays"] = true
}

--
function gTable.OnLoad(...)
    if GDB.initializedDate == nil then
        DEFAULT_CHAT_FRAME:AddMessage("|cff965573" .. "Guildster: " .. "|r" .. "Setting initialization Date")
        GDB.initializedDate = gTable.GetCurrentTime()
    end
end

gTable.tempDB = {}
gTable.sortedDB = {}

function gTable.PopulateScrollFrame(...)
    local n = 1

    -- Fix for under <= 3 results
    local blankButton = {}
    if (#gTable.sortedDB < 4 and #gTable.sortedDB > 0) then
        for i = #gTable.sortedDB, 3 do
            table.insert(gTable.sortedDB, blankButton)
        end
    end

    for k, v in pairs(gTable.sortedDB) do
        gTable.tempDB[n] = k
        n = n + 1
    end
    n = n - 1

    FauxScrollFrame_Update(gScrollFrame, n, gTable.numButtons, gTable.buttonHeight)

    for index = 1, gTable.numButtons do
        local offset = index + FauxScrollFrame_GetOffset(gScrollFrame)
        local gButton = gScrollFrame.buttons[index]
        gButton.index = offset
        if (offset <= n) then
            tempOffset = gTable.sortedDB[gTable.tempDB[offset]]

            -- Fix for under <= 3 results
            if (tempOffset.Name == nil) then
                gButton:Hide()
            else
                gButton:Show()
            end

            gButton.nameText:SetText(tempOffset.Name)
            gButton.membersText:SetText(tempOffset.Members)
            gButton.acvText:SetText(tempOffset.AchvPts)
            gButton.commentText:SetText(tempOffset.Comment)
            if (gTable.CheckForRecentActivity(tempOffset.DateUpdated) ~= "olderThan30") then
                gButton.dateText:SetText("Recently Updated")
            else
                gButton.dateText:SetText(nil)
            end

            SetLargeGuildTabardTextures(nil, gButton.tabardEmblem, gButton.tabard, gButton.border, tempOffset.Tabard)

            if (tempOffset.Name ~= nil) then
                for k, v in pairs(tempOffset.Filters) do
                    if v == true then
                        gButton[k]:Show()
                    else
                        gButton[k]:Hide()
                    end
                end
            end

            -- Highlighting / Selecting
            gScrollFrame:SetScript(
                "OnVerticalScroll",
                function(self, offset)
                    gTable.selection = nil
                    Guildster_Req_Button:Disable()
                    gTable.sFrame:Hide()
                    FauxScrollFrame_OnVerticalScroll(self, offset, gTable.buttonHeight, gTable.PopulateScrollFrame)
                end
            )
            gButton:SetScript(
                "OnClick",
                function(self)
                    if (selectTexture:IsShown()) then
                        gTable.sFrame:Hide()
                        gTable.selection = nil
                    end
                    selectTexture:SetPoint("TOPLEFT", gButton, "TOPLEFT", 0, 4)
                    gTable.sFrame:Show()
                    gTable.selection = gButton.nameText:GetText()

                    local _, numAppsRemaining = GetNumGuildMembershipRequests()
                    if (gTable.selection ~= nil and numAppsRemaining > 0) then
                        Guildster_Req_Button:Enable()

                        -- Getting the indexes of the items in the list
                        ToggleGuildFrame(1)
                        ToggleGuildFrame(1)
                        local guildIndex = {}
                        local selectedIndex = nil
                        local numGuilds = GetNumRecruitingGuilds()
                        for i = 1, numGuilds do
                            local name, level, numMembers, achPoints, comment, cached, requestPending =
                                GetRecruitingGuildInfo(i)
                            table.insert(guildIndex, name)
                        end
                        for k, v in pairs(guildIndex) do
                            if (gTable.selection == v) then
                                selectedIndex = k
                                break
                            end
                        end
                        SetRecruitingGuildSelection(k)
                    else
                        Guildster_Req_Button:Disable()
                    end
                end
            )

            Guildster_Req_Button:SetScript(
                "OnClick",
                function(self)
                    StaticPopupSpecial_Show(GuildFinderRequestMembershipFrame)
                    local name, level = GetRecruitingGuildInfo(GetRecruitingGuildSelection())
                    GuildFinderRequestMembershipFrameGuildName:SetText(gTable.selection)
                    GuildFinderRequestMembershipEditBox:SetText(GetLookingForGuildComment())
                end
            )
        else
            gButton:Hide()
        end
    end
end

function gTable.WriteToDB(name, numMembers, achPoints, comment, tabardInfo, filterInfo, dateUpdated)
    GuildsterDB[name] = {
        Name = name,
        Members = numMembers,
        AchvPts = achPoints,
        Comment = comment,
        Tabard = tabardInfo,
        Filters = filterInfo,
        DateUpdated = dateUpdated
    }
end

function gTable.GetGuildInfo(self, event, arg1, arg2, arg3, ...)
    gTable.OnLoad()

    local numGuilds = GetNumRecruitingGuilds() -- Number of Guilds in the Recruitment Frame

    local newGuilds = 0
    local updatedGuilds = 0
    for i = 1, numGuilds do
        local name, level, numMembers, achPoints, comment, cached, requestPending = GetRecruitingGuildInfo(i)
        local bQuest, bDungeon, bRaid, bPvP, bRP, bWeekdays, bWeekends, bTank, bHealer, bDamage =
            GetRecruitingGuildSettings(i)
        local filterInfo = {
            Questing = bQuest,
            Dungeons = bDungeon,
            Raiding = bRaid,
            PvP = bPvP,
            RP = bRP,
            Weekdays = bWeekdays,
            Weekends = bWeekends,
            Tank = bTank,
            Healer = bHealer,
            Damage = bDamage
        }
        local tabardInfo = {GetRecruitingGuildTabardInfo(i)}
        local dateUpdated = gTable.GetCurrentTime()
        if (GuildsterDB[name]) then --Check if the DB cotains this entry already
            if (GuildsterDB[name].Comment ~= comment or numMembers > GuildsterDB[name].Members) then --Check for difference
                gTable.WriteToDB(name, numMembers, achPoints, comment, tabardInfo, filterInfo, dateUpdated)
                updatedGuilds = updatedGuilds + 1
            else
                -- Do some stuff eventually if it's "too old"
            end
        else
            gTable.WriteToDB(name, numMembers, achPoints, comment, tabardInfo, filterInfo, dateUpdated)
            newGuilds = newGuilds + 1
        end
        -- end
    end

    if newGuilds > 0 then
        DEFAULT_CHAT_FRAME:AddMessage("|cff965573" .. "Guildster: " .. "|r" .. newGuilds .. " new guilds listed.")
    end
    if updatedGuilds > 0 then
        DEFAULT_CHAT_FRAME:AddMessage("|cff965573" .. "Guildster: " .. "|r" .. updatedGuilds .. " guilds updated.")
    end
end

function gTable.SortOption(...)
    if (GDB.sortOption ~= nil) then
        if GDB.sortOption == "name" then
            gTable.OnButtonClick(gTable.CompareName)
        elseif GDB.sortOption == "members" then
            gTable.OnButtonClick(gTable.CompareMembers)
        elseif GDB.sortOption == "achievements" then
            gTable.OnButtonClick(gTable.CompareAchvs)
        elseif GDB.sortOption == "date" then
            gTable.OnButtonClick(gTable.CompareDates)
        end
    else
        gTable.OnButtonClick(gTable.CompareName)
    end
end

-- Comparison Sorting
function gTable.OnButtonClick(sortType, ...)
    gTable.CreateTempDB()
    table.sort(gTable.sortedDB, sortType)
    gTable.PopulateScrollFrame()
end

function gTable.CompareName(a, b)
    GDB.sortOption = "name"
    return a.Name < b.Name
end

function gTable.CompareMembers(a, b)
    local result = a.Members > b.Members
    if a.Members == b.Members then
        result = a.Name < b.Name
    end
    GDB.sortOption = "members"
    return result
end

function gTable.CompareAchvs(a, b)
    local result = a.AchvPts > b.AchvPts
    if a.AchvPts == b.AchvPts then
        result = a.Name < b.Name
    end
    GDB.sortOption = "achievements"
    return result
end

function gTable.CompareDates(a, b)
    local result = a.DateUpdated > b.DateUpdated
    if a.DateUpdated == b.DateUpdated then
        result = a.Name < b.Name
    end
    GDB.sortOption = "date"
    return result
end

function gTable.CreateTempDB()
    table.wipe(gTable.sortedDB)

    local tempFilters = {}
    for k, v in pairs(gTable.filterTypes) do
        if (v == true) then
            table.insert(tempFilters, k)
        end
    end

    local u = 0
    for game, value in pairs(GuildsterDB) do
        if string.find(value.Name, gTable.IgnoreCase(gTable.searchValue)) then
            if value.Members >= gTable.memberFilterLow and value.Members <= gTable.memberFilterHigh then
                if gTable.updatedFilter == true and gTable.CheckForRecentActivity(value.DateUpdated) ~= "olderThan30" then
                    local pos = 1
                    for k, v in pairs(tempFilters) do
                        if (value.Filters[v] == false) then
                            pos = pos + 1
                            break
                        elseif value.Filters[v] == true and pos == #tempFilters then
                            gTable.InsertToTempDB(value)
                            u = u + 1
                        else
                            pos = pos + 1
                        end
                    end
                else
                    local pos = 1
                    for k, v in pairs(tempFilters) do
                        if (value.Filters[v] == false) then
                            pos = pos + 1
                            break
                        elseif value.Filters[v] == true and pos == #tempFilters then
                            gTable.InsertToTempDB(value)
                            u = u + 1
                        else
                            pos = pos + 1
                        end
                    end
                end
            else
                -- Bup
            end
        else
            -- Bup2
        end
    end
    gTable.resultsFound = u
end

function gTable.InsertToTempDB(value)
    table.insert(
        gTable.sortedDB,
        {
            Name = value.Name,
            Members = value.Members,
            AchvPts = value.AchvPts,
            Comment = value.Comment,
            Tabard = value.Tabard,
            Filters = value.Filters,
            DateUpdated = value.DateUpdated
        }
    )
end

function gTable.GetCurrentTime(...)
    local curTime = GetServerTime()
    return curTime
end

function gTable.CheckForRecentActivity(oldDate)
    if oldDate == nil then
        return nil
    elseif (oldDate + 1209600) < GetServerTime() then
        return "olderThan30"
    end
end

function gTable.IgnoreCase(s)
    s =
        string.gsub(
        s,
        "%a",
        function(c)
            return string.format("[%s%s]", string.lower(c), string.upper(c))
        end
    )
    return s
end

function gTable.StartGuildSearch()
    for i = 1, 10 do
        SetLookingForGuildSettings(i, true)
    end
    for i = 1, 10 do
        SetLookingForGuildSettings(i, true)
    end

    DEFAULT_CHAT_FRAME:AddMessage("|cff965573" .. "Guildster: " .. "|r" .. "v" .. version .. " loaded")
    DEFAULT_CHAT_FRAME:AddMessage("Use /guildster to access the main menu.")
    RequestGuildMembershipList()
    gTable.GetGuildInfo()
end

local z = CreateFrame("Frame")
z:RegisterEvent("PLAYER_LOGIN")
z:SetScript(
    "OnEvent",
    function(self)
        gTable.StartGuildSearch()
        gTable.GetGuildInfo()
    end
)
