local guildster, gTable = ...

function gTable.CreateFilterFrame(...)
    -- Top Filtering Frame
    gTable.filterFrame = CreateFrame("Frame", "Guildster_Filter_Frame", gTable.frameBG)
    gTable.filterFrame:SetSize(750, 80)
    gTable.filterFrame:SetPoint("TOPLEFT", gTable.header, "BOTTOMLEFT")
    gTable.filterFrame:SetFrameStrata("HIGH")

    gTable.filterFrame.bg = gTable.filterFrame:CreateTexture("nil", "BACKGROUND")
    gTable.filterFrame.bg:SetColorTexture(0, 0, 0, 0.2)
    gTable.filterFrame.bg:SetSize(750, 80)
    gTable.filterFrame.bg:SetPoint("TOPLEFT", gTable.filterFrame, "TOPLEFT")

    gTable.filterFrame.divider = gTable.filterFrame:CreateTexture(nil, "ARTWORK")
    gTable.filterFrame.divider:SetPoint("TOPLEFT", gTable.filterFrame, "BOTTOMLEFT", -2, 2)
    gTable.filterFrame.divider:SetWidth(gTable.frameBG:GetWidth() - 6)
    gTable.filterFrame.divider:SetHeight(9)
    gTable.filterFrame.divider:SetTexture("Interface\\BankFrame\\HorizShadow")
    gTable.filterFrame.divider:SetAlpha(0.3)

    local menuFrame = CreateFrame("Frame", "Guildster_Sort_Frame", UIParent, "UIDropDownMenuTemplate")

    local sortMenu = {
        {text = "Sorting Options", isTitle = true},
        {
            text = "Name",
            func = function()
                gTable.OnButtonClick(gTable.CompareName)
            end
        },
        {
            text = "Members",
            func = function()
                gTable.OnButtonClick(gTable.CompareMembers)
            end
        },
        {
            text = "Achievement Points",
            func = function()
                gTable.OnButtonClick(gTable.CompareAchvs)
            end
        },
        {
            text = "Date Updated",
            func = function()
                gTable.OnButtonClick(gTable.CompareDates)
            end
        },
        {
            text = "Close Menu",
            func = function()
                menuFrame:Hide()
            end
        }
    }

    local membersSliderLow =
        CreateFrame("Slider", "Guildster_Members_Slider_Low", gTable.filterFrame, "OptionsSliderTemplate")
    membersSliderLow:SetWidth(100)
    membersSliderLow:SetHeight(12)
    membersSliderLow:SetPoint("TOPLEFT", gTable.filterFrame, "TOPLEFT", 20, -20)
    membersSliderLow:SetOrientation("HORIZONTAL")
    _G[membersSliderLow:GetName() .. "Text"]:SetText("Members Filter Low")
    _G[membersSliderLow:GetName() .. "Low"]:SetText("1")
    _G[membersSliderLow:GetName() .. "High"]:SetText("999")
    membersSliderLow:Show()
    membersSliderLow:SetMinMaxValues(1, 999)
    membersSliderLow:SetScript(
        "OnValueChanged",
        function(self)
            local value = tonumber(string.format("%.0f", self:GetValue()))
            if (Guildster_Members_Slider_High:GetValue() <= value) then
                Guildster_Members_Slider_High:SetValue(value + 1)
            end
            gTable.memberFilterLow = value
            membersSliderLow.Ebox:SetText(value)
            membersSliderLow.Ebox:ClearFocus()
            gTable.SortOption(guildster, gTable)
        end
    )

    membersSliderLow.Ebox = CreateFrame("Editbox", nil, membersSliderLow, "InputBoxTemplate")
    membersSliderLow.Ebox:SetSize(35, 30)
    membersSliderLow.Ebox:SetAutoFocus(false)
    membersSliderLow.Ebox:SetNumeric(true)
    membersSliderLow.Ebox:SetPoint("LEFT", 120, 0)
    membersSliderLow.Ebox:SetScript(
        "OnEditFocusLost",
        function(self)
            if self:GetText() ~= "" then
                local value = tonumber(string.format("%.0f", self:GetText()))
                if (Guildster_Members_Slider_High:GetValue() <= value) then
                    Guildster_Members_Slider_High:SetValue(value + 1)
                end
                gTable.memberFilterLow = value
                membersSliderLow:SetValue(value)
                gTable.SortOption(guildster, gTable)
            end
        end
    )

    local membersSliderHigh =
        CreateFrame("Slider", "Guildster_Members_Slider_High", gTable.filterFrame, "OptionsSliderTemplate")
    membersSliderHigh:SetWidth(100)
    membersSliderHigh:SetHeight(12)
    membersSliderHigh:SetPoint("TOP", membersSliderLow, "BOTTOM", 0, -22)
    membersSliderHigh:SetOrientation("HORIZONTAL")
    _G[membersSliderHigh:GetName() .. "Text"]:SetText("Members Filter High")
    _G[membersSliderHigh:GetName() .. "Low"]:SetText("1")
    _G[membersSliderHigh:GetName() .. "High"]:SetText("999")
    membersSliderHigh:Show()
    membersSliderHigh:SetMinMaxValues(1, 999)
    membersSliderHigh:SetScript(
        "OnValueChanged",
        function(self)
            local value = tonumber(string.format("%.0f", self:GetValue()))
            if (Guildster_Members_Slider_Low:GetValue() >= value) then
                Guildster_Members_Slider_Low:SetValue(value - 1)
            end

            gTable.memberFilterHigh = value
            membersSliderHigh.Ebox:SetText(value)
            membersSliderHigh.Ebox:ClearFocus()
            gTable.SortOption(guildster, gTable)
        end
    )

    membersSliderHigh.Ebox = CreateFrame("Editbox", "Guildster_SliderHigh_Box", membersSliderHigh, "InputBoxTemplate")
    membersSliderHigh.Ebox:SetSize(35, 30)
    membersSliderHigh.Ebox:SetAutoFocus(false)
    membersSliderHigh.Ebox:SetNumeric(true)
    membersSliderHigh.Ebox:SetPoint("LEFT", 120, 0)
    membersSliderHigh.Ebox:SetEnabled(true)
    membersSliderHigh.Ebox:SetScript(
        "OnEditFocusLost",
        function(self)
            if self:GetText() ~= "" then
                local value = tonumber(string.format("%.0f", self:GetText()))
                if (Guildster_Members_Slider_Low:GetValue() >= value) then
                    Guildster_Members_Slider_Low:SetValue(value - 1)
                end
                gTable.memberFilterHigh = value
                membersSliderHigh:SetValue(value)
                gTable.SortOption(guildster, gTable)
            end
        end
    )

    -- Interests
    gTable.filterFrame.interests = gTable.filterFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    gTable.filterFrame.interests:SetPoint("TOPLEFT", gTable.filterFrame, "TOP", -190, -7)
    gTable.filterFrame.interests:SetText("Interests")

    gTable.filterFrame.intDiv = gTable.filterFrame:CreateTexture(nil, "ARTWORK")
    gTable.filterFrame.intDiv:SetPoint("TOPLEFT", gTable.filterFrame.interests, "BOTTOMLEFT", 0, 0)
    gTable.filterFrame.intDiv:SetWidth(200)
    gTable.filterFrame.intDiv:SetHeight(6)
    gTable.filterFrame.intDiv:SetTexture("Interface\\BankFrame\\HorizShadow")
    gTable.filterFrame.intDiv:SetAlpha(0.3)

    gTable.filterFrame.questing = CreateFrame("CheckButton", "gbQd", gTable.filterFrame, "UICheckButtonTemplate")
    gTable.filterFrame.questing:SetPoint("TOPLEFT", gTable.filterFrame.interests, "BOTTOMLEFT", 18, 0)
    gTable.filterFrame.questing:SetSize(24, 24)
    gTable.filterFrame.questing:SetChecked(true)
    _G[gTable.filterFrame.questing:GetName() .. "Text"]:SetText("Questing")
    gTable.filterFrame.questing:SetScript(
        "OnClick",
        function(self)
            local value = gTable.filterFrame.questing:GetChecked()
            gTable.filterTypes["Questing"] = value
            gTable.SortOption(guildster, gTable)
        end
    )

    gTable.filterFrame.dungeons = CreateFrame("CheckButton", "gbDd", gTable.filterFrame, "UICheckButtonTemplate")
    gTable.filterFrame.dungeons:SetPoint("LEFT", gTable.filterFrame.questing, "RIGHT", 60, 0)
    gTable.filterFrame.dungeons:SetSize(24, 24)
    gTable.filterFrame.dungeons:SetChecked(true)
    _G[gTable.filterFrame.dungeons:GetName() .. "Text"]:SetText("Dungeons")
    gTable.filterFrame.dungeons:SetScript(
        "OnClick",
        function(self)
            local value = gTable.filterFrame.dungeons:GetChecked()
            gTable.filterTypes["Dungeons"] = value
            gTable.SortOption(guildster, gTable)
        end
    )

    gTable.filterFrame.raids = CreateFrame("CheckButton", "gbRd", gTable.filterFrame, "UICheckButtonTemplate")
    gTable.filterFrame.raids:SetPoint("TOPLEFT", gTable.filterFrame.questing, "BOTTOMLEFT", 0, 6)
    gTable.filterFrame.raids:SetSize(24, 24)
    gTable.filterFrame.raids:SetChecked(true)
    _G[gTable.filterFrame.raids:GetName() .. "Text"]:SetText("Raids")
    gTable.filterFrame.raids:SetScript(
        "OnClick",
        function(self)
            local value = gTable.filterFrame.raids:GetChecked()
            gTable.filterTypes["Raiding"] = value
            gTable.SortOption(guildster, gTable)
        end
    )

    gTable.filterFrame.pvp = CreateFrame("CheckButton", "gbpvpd", gTable.filterFrame, "UICheckButtonTemplate")
    gTable.filterFrame.pvp:SetPoint("TOPLEFT", gTable.filterFrame.dungeons, "BOTTOMLEFT", 0, 6)
    gTable.filterFrame.pvp:SetSize(24, 24)
    gTable.filterFrame.pvp:SetChecked(true)
    _G[gTable.filterFrame.pvp:GetName() .. "Text"]:SetText("PvP")
    gTable.filterFrame.pvp:SetScript(
        "OnClick",
        function(self)
            local value = gTable.filterFrame.pvp:GetChecked()
            gTable.filterTypes["PvP"] = value
            gTable.SortOption(guildster, gTable)
        end
    )

    gTable.filterFrame.rp = CreateFrame("CheckButton", "gbrpd", gTable.filterFrame, "UICheckButtonTemplate")
    gTable.filterFrame.rp:SetPoint("TOPLEFT", gTable.filterFrame.raids, "BOTTOMLEFT", 0, 6)
    gTable.filterFrame.rp:SetSize(24, 24)
    gTable.filterFrame.rp:SetChecked(true)
    _G[gTable.filterFrame.rp:GetName() .. "Text"]:SetText("Role Playing")
    gTable.filterFrame.rp:SetScript(
        "OnClick",
        function(self)
            local value = gTable.filterFrame.rp:GetChecked()
            gTable.filterTypes["RP"] = value
            gTable.SortOption(guildster, gTable)
        end
    )

    -- Availability
    gTable.filterFrame.avail = gTable.filterFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    gTable.filterFrame.avail:SetPoint("TOPLEFT", gTable.filterFrame.interests, "TOPRIGHT", 160, 0)
    gTable.filterFrame.avail:SetText("Availability")

    gTable.filterFrame.availDiv = gTable.filterFrame:CreateTexture(nil, "ARTWORK")
    gTable.filterFrame.availDiv:SetPoint("TOPLEFT", gTable.filterFrame.avail, "BOTTOMLEFT", 0, 0)
    gTable.filterFrame.availDiv:SetWidth(200)
    gTable.filterFrame.availDiv:SetHeight(6)
    gTable.filterFrame.availDiv:SetTexture("Interface\\BankFrame\\HorizShadow")
    gTable.filterFrame.availDiv:SetAlpha(0.3)

    gTable.filterFrame.days = CreateFrame("CheckButton", "gbdaysd", gTable.filterFrame, "UICheckButtonTemplate")
    gTable.filterFrame.days:SetPoint("TOPLEFT", gTable.filterFrame.avail, "BOTTOMLEFT", 18, 0)
    gTable.filterFrame.days:SetSize(24, 24)
    gTable.filterFrame.days:SetChecked(true)
    _G[gTable.filterFrame.days:GetName() .. "Text"]:SetText("Weekdays")
    gTable.filterFrame.days:SetScript(
        "OnClick",
        function(self)
            local value = gTable.filterFrame.days:GetChecked()
            gTable.filterTypes["Weekdays"] = value
            gTable.SortOption(guildster, gTable)
        end
    )

    gTable.filterFrame.ends = CreateFrame("CheckButton", "gbendsd", gTable.filterFrame, "UICheckButtonTemplate")
    gTable.filterFrame.ends:SetPoint("LEFT", gTable.filterFrame.days, "RIGHT", 60, 0)
    gTable.filterFrame.ends:SetSize(24, 24)
    gTable.filterFrame.ends:SetChecked(true)
    _G[gTable.filterFrame.ends:GetName() .. "Text"]:SetText("Weekends")
    gTable.filterFrame.ends:SetScript(
        "OnClick",
        function(self)
            local value = gTable.filterFrame.ends:GetChecked()
            gTable.filterTypes["Weekends"] = value
            gTable.SortOption(guildster, gTable)
        end
    )

    searchBox = CreateFrame("Editbox", "Guildster_Search_Box", gTable.filterFrame, "InputBoxTemplate")
    searchBox:SetSize(140, 30)
    searchBox:SetAutoFocus(false)
    searchBox:SetPoint("BOTTOMRIGHT", gTable.filterFrame, "BOTTOMRIGHT", -158, 6)
    searchBox:SetScript(
        "OnTextChanged",
        function(self)
            local value = self:GetText()
            gTable.searchValue = value
            gTable.SortOption(guildster, gTable)
        end
    )

    sText = searchBox:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    sText:SetPoint("RIGHT", searchBox, "LEFT", -8, 0)
    sText:SetText("Search")

    local sortButton = CreateFrame("Button", "Guildster_Sort_Button", gTable.filterFrame, "UIPanelButtonTemplate")
    sortButton:SetSize(120, 22)
    sortButton:SetText("Sorting Options")
    sortButton:SetPoint("BOTTOMRIGHT", gTable.filterFrame, "BOTTOMRIGHT", -10, 20)
    sortButton:SetScript(
        "OnEnter",
        function(self)
            GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
            GameTooltip:ClearLines()
            GameTooltip:SetText("Sort the Guilds by parameters", 1, 1, .10, 1, 1)
            GameTooltip:Show()
        end
    )
    sortButton:SetScript(
        "OnLeave",
        function(self)
            GameTooltip:Hide()
        end
    )
    sortButton:SetScript(
        "OnClick",
        function()
            EasyMenu(sortMenu, menuFrame, "cursor", 0, 0, "MENU")
        end
    )

    local guildster_date =
        CreateFrame("CheckButton", "Guildster_Date_Filter", gTable.filterFrame, "UICheckButtonTemplate")
    guildster_date:SetPoint("BOTTOMLEFT", sortButton, "TOPLEFT", -2, -2)
    guildster_date:SetSize(24, 24)
    _G[guildster_date:GetName() .. "Text"]:SetText("Recently Updated")
    guildster_date:SetScript(
        "OnEnter",
        function(self)
            GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
            GameTooltip:ClearLines()
            GameTooltip:SetText("Only checks guilds which have been updated recently.", 1, 1, .10, 1, 1)
            GameTooltip:Show()
        end
    )
    guildster_date:SetScript(
        "OnLeave",
        function(self)
            GameTooltip:Hide()
        end
    )
    guildster_date:SetScript(
        "OnClick",
        function(self)
            local value = guildster_date:GetChecked()
            gTable.updatedFilter = value
            gTable.SortOption(guildster, gTable)
        end
    )

    local results = sortButton:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    results:SetPoint("TOP", sortButton, "BOTTOM", 0, -4)

    local g = CreateFrame("Frame")
    g:SetScript(
        "OnUpdate",
        function(self)
            if gTable.resultsFound ~= nil then
                results:SetText("Found " .. gTable.resultsFound .. " guild(s)")
            else
                results:SetText(nil)
            end
        end
    )

    gTable.sFrame = CreateFrame("Frame")
    gTable.sFrame:SetFrameStrata("HIGH")
    gTable.sFrame:Hide()

    selectTexture = gTable.sFrame:CreateTexture(nil, "OVERLAY")
    selectTexture:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
    selectTexture:SetColorTexture(1, 0, 0, 0.1)
    selectTexture:SetSize(528, 100)
end
